use chromium::rocket::Chromium;
use log::debug;
use rocket::{Rocket, Build, routes};
use tools::rocket::TracingFairing;
use tracing::Level;
use tracing_subscriber::EnvFilter;
use tracing_subscriber::prelude::*;
pub mod routes;
mod chromium;
mod tools;


use rocket::http::Status;
use rocket::request::FromRequest;
use rocket::request::Outcome;
use rocket::serde::{json::Json, Serialize};
use rocket::{
    fairing::{Fairing, Info, Kind},
    Data, Request, Response,
};


use tracing::{info_span, Span};
use tracing_log::LogTracer;

use tracing_subscriber::registry::LookupSpan;
use uuid::Uuid;
use yansi::Paint;

    use opentelemetry::trace::TracerProvider as _;
    use opentelemetry_sdk::trace::TracerProvider;
    use opentelemetry_stdout as stdout;
    use tracing::{error, span};
    use tracing_subscriber::layer::SubscriberExt;
    use tracing_subscriber::Registry;
    
  
    use opentelemetry::{
        metrics::MeterProvider as _,
        trace::{Tracer, TracerProvider as _},
        KeyValue,
    };
    use opentelemetry_sdk::{
        metrics::{PeriodicReader, MeterProvider},
        runtime,
    };


#[derive(PartialEq, Eq, Debug, Clone, Copy)]
pub enum LogLevel {
    /// Only shows errors and warnings: `"critical"`.
    Critical,
    /// Shows errors, warnings, and some informational messages that are likely
    /// to be relevant when troubleshooting such as configuration: `"support"`.
    Support,
    /// Shows everything except debug and trace information: `"normal"`.
    Normal,
    /// Shows everything: `"debug"`.
    Debug,
    /// Shows nothing: "`"off"`".
    Off,
}


pub fn filter_layer(level: LogLevel) -> EnvFilter {
    let filter_str = match level {
        LogLevel::Critical => "warn,hyper=off,rustls=off",
        LogLevel::Support => "warn,rocket::support=info,hyper=off,rustls=off",
        LogLevel::Normal => "info,hyper=off,rustls=off",
        LogLevel::Debug => "trace",
        LogLevel::Off => "off",
    };

    tracing_subscriber::filter::EnvFilter::try_new(filter_str).expect("filter string must parse")
}


pub fn json_logging_layer<
    S: for<'a> tracing_subscriber::registry::LookupSpan<'a> + tracing::Subscriber,
>() -> impl tracing_subscriber::Layer<S> {
    //Paint::disable();

    tracing_subscriber::fmt::layer()
        .json()
        // Configure the formatter to use `print!` rather than
        // `stdout().write_str(...)`, so that logs are captured by libtest's test
        // capturing.
        .with_test_writer()
}


pub fn default_logging_layer<S>() -> impl tracing_subscriber::Layer<S>
where
    S: tracing::Subscriber,
    S: for<'span> LookupSpan<'span>,
{
    let field_format = tracing_subscriber::fmt::format::debug_fn(|writer, field, value| {
        // We'll format the field name and value separated with a colon.
        if field.name() == "message" {
            write!(writer, "{:?}", Paint::new(value).bold())
        } else {
            write!(writer, "{}: {:?}", field, Paint::default(value).bold())
        }
    })
    .delimited(", ")
    .display_messages();

    tracing_subscriber::fmt::layer()
        .fmt_fields(field_format)
        // Configure the formatter to use `print!` rather than
        // `stdout().write_str(...)`, so that logs are captured by libtest's test
        // capturing.
        .with_test_writer()
}


pub fn rocket_builder() -> Rocket<Build> {
    //let tracer_provider = init_trace();

    rocket::build()
        .attach(TracingFairing {})
        .attach(Chromium::ignite())
        .attach(tools::rocket::RequestTimer)
        .mount("/", routes![
            routes::pdf::render_pdf_cv,
        ])
}